import os
def create_file(content, file_name, file_extension, folder=None):
    if folder:
        if file_extension:
            file_path = "{}/{}.{}".format(folder, file_name, file_extension)
        else:
            file_path = "{}/{}".format(folder, file_name)
    else:
        if file_extension:
            file_path = "{}.{}".format(file_name, file_extension)
        else:
            file_path = "{}".format(file_name)
    # write to file
    try:
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        with open(file_path, "wb") as f:
            f.write(content.encode('utf8'))
    except Exception as e:
        print(e)
        return None
    return file_path



# coding=utf-8

import re
from jinja2 import Environment, PackageLoader


class ViewModel(object):

    class Property(object):
        def __init__(self, name, type_name):
            super(ViewModel.Property, self).__init__()
            self.name = name
            self.type_name = type_name

        def __str__(self):
            return "let {}: Stream<{}>".format(self.name, self.type_name)

    def __init__(self, vm_text):
        super(ViewModel, self).__init__()
        self.vm_text = vm_text

    @property
    def view_model_name(self):
        try:
            regex = re.compile(r'(?:struct|extension) (\w+)ViewModel')
            mo = regex.search(self.vm_text)
            return mo.group(1)
        except Exception:
            print("The ViewModel in the pasteboard is invalid.")
            exit(1)

    @property
    def properties(self):
        try:
            str = self.vm_text
            input_block_regex = re.compile("struct Input {([^}]+)")
            input_block = input_block_regex.search(str).group(1)
            input_properties_regex = re.compile(r'let (\w+): Stream<([^>]+)>')
            input_properties = [
                ViewModel.Property(p[0], p[1])
                for p in input_properties_regex.findall(input_block)
            ]
            output_block_regex = re.compile("struct Output {([^}]+)")
            output_block = output_block_regex.search(str).group(1)
            output_properties_regex = re.compile(r'let (\w+): Stream<([^>]+)>')
            output_properties = [
                ViewModel.Property(p[0], p[1])
                for p in output_properties_regex.findall(output_block)
            ]
            return (input_properties, output_properties)
        except Exception:
            print("The ViewModel in the pasteboard is invalid.")
            exit(1)


class UnitTest(ViewModel):

    def create_tests(self):
        input_properties, output_properties = self.properties
        env = Environment(
            loader=PackageLoader('ptfgen_templates', 'commands'),
            trim_blocks=True,
            lstrip_blocks=True
        )
        template = env.get_template("unit_test.dart")
        content = template.render(
            name=self.view_model_name,
            input_properties=input_properties,
            output_properties=output_properties
        )
        return content


class BindViewModel(ViewModel):

    def create_bind_view_model(self):
        input_properties, output_properties = self.properties
        env = Environment(
            loader=PackageLoader('ptfgen_templates', 'commands'),
            trim_blocks=True,
            lstrip_blocks=True
        )
        template = env.get_template("bindviewmodel.dart")
        content = template.render(
            name=self.view_model_name,
            input_properties=input_properties,
            output_properties=output_properties
        )
        return content



# coding=utf-8

import os
from datetime import datetime
from jinja2 import Environment, PackageLoader
from .str_helpers import upper_first_letter, lower_first_letter
from .file_helpers import create_file
import re

class ProjectInfo:

    def __init__(self, project, developer, company, project_id):
        self.project = project
        self.developer = developer
        self.company = company
        self.project_id = project_id


class Template(object):

    class TemplateType:
        BASE = 'base'

    # =============== BaseTemplate ===============

    class BaseTemplate(object):
        def __init__(self, options, name, package_name):
            self.name = name
            self.short_name = self.create_file_name(self.name)
            self.package_name = package_name

            self.output_path = '.'

            self.env = Environment(
                loader=PackageLoader('ptfgen_templates', 'base'),
                trim_blocks=True,
                lstrip_blocks=True
            )

            self.file_header_env = Environment(
                loader=PackageLoader('ptfgen_templates', 'commands'),
                trim_blocks=True,
                lstrip_blocks=True
            )

        def create_files(self):
            print('Successfully created files:')
            # Register
            self._register_assembler()
            # Base
            self._create_view_model()
            self._create_navigator()
            self._create_use_case()
            self._create_assembler()
            self._create_view()
            # Mock
            self._create_use_case_scene_mock()
            # Test
            self._create_view_model_tests()
            self._create_use_case_mock()
            self._create_navigator_mock()

        def _make_dirs(self, path):
            current_directory = os.getcwd() if self.output_path == '.' else self.output_path
            self._make_dir(current_directory, path)

        def _make_dir(self, current_directory, new_directory_name):
            directory = os.path.join(current_directory,
                                     r'{}'.format(new_directory_name))
            try:
                os.makedirs(directory)
            except Exception as e:
                # print(e)
                pass
            return directory

        def _create_file_from_template(self,
                                       class_name,
                                       file_extension="dart",
                                       template_file=None,
                                       include_root_folder=True,
                                       output_path=None):
            if template_file is None:
                if file_extension:
                    template_file = '{}.{}'.format(class_name, file_extension)
                else:
                    template_file = class_name[len(self.short_name):]

            file_name = '{}.{}'.format(class_name, file_extension) \
                if file_extension is not None else class_name

            template = self.env.get_template(template_file)
            content = self._content_from_template(template)

            if output_path is None:
                output_path = './'

            if output_path.endswith('/'):
                output_path = output_path[:-1]

            if include_root_folder:
                folder = './{}/{}'.format(output_path, self.short_name)
            else:
                folder = './{}'.format(output_path)

            file_path = create_file(
                content=content,
                file_name=class_name,
                file_extension=file_extension,
                folder=folder
            )

            if file_path is not None:
                print('    {}'.format(file_path))
        
        def is_camel_case(self, s):
            return s != s.lower() and s != s.upper() and "_" not in s

        def create_file_name(self, class_name):
            if self.is_camel_case(class_name):
                upper_case_letters = re.findall('[A-Z][^A-Z]*', class_name)
                return '_'.join(upper_case_letters).lower()
            else:
                print('Invalid filename. Filename need to be camel string.\nFor example: MyCamelCase')
                exit(1)

        def _content_from_template(self, template):
            return template.render(
                name=self.name,
                name_lower=self.short_name,
                package_name=self.package_name
            )

        def _create_view_model(self):
            scene_name = 'viewmodel'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='lib/scenes'
            )

        def _create_navigator(self):
            scene_name = 'navigator'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='lib/scenes'
            )

        def _create_use_case(self):
            scene_name = 'usecase'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='lib/scenes'
            )

        def _create_assembler(self):
            scene_name = 'assembler'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='lib/scenes'
            )

        def _create_view(self):
            scene_name = 'view'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='lib/scenes'
            )

        # =============== Mock ===============

        def _create_use_case_scene_mock(self):
            scene_name = 'usecase_mock'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                include_root_folder=False,
                output_path='lib/mock'
            )

        # =============== UnitTests ===============

        def _create_view_model_tests(self):
            scene_name = 'viewmodel_test'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='test/unit_test'
            )

        def _create_use_case_mock(self):
            scene_name = 'usecase_mock'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='test/unit_test'
            )

        def _create_navigator_mock(self):
            scene_name = 'navigator_mock'
            self._create_file_from_template(
                class_name=self.short_name + '_' + scene_name,
                template_file='{}.dart'.format(scene_name),
                output_path='test/unit_test'
            )

        # =============== Other ===============

        def _register_assembler(self):
            try:
                path = 'lib/assembler.dart'
                with open(path, 'r') as file :
                    filedata = file.read()
                import_file = "import 'package:{}/scenes/{}/{}_assembler.dart';\n".format(self.package_name, self.short_name, self.short_name)
                filedata =  import_file + filedata
                if 'with' in filedata:
                    filedata = filedata.replace(' {}', ', {}Assembler {{}}'.format(self.name))
                else:
                    filedata = filedata.replace(' {}', 'with {}Assembler {{}}'.format(self.name))
                with open(path, 'w') as file:
                    file.write(filedata)
                print('    ./lib/assembler.dart')
            except (IOError, OSError) as e:
                print("Can't find assembler.dart")
                exit(1)


# coding=utf-8

from .pb import pasteboard_write
from .vm import UnitTest
from .command import Command


class UnitTestCommand(Command):
    def __init__(self, vm_text):
        super(UnitTestCommand, self).__init__()
        self.vm_text = vm_text

    def create_tests(self, print_result):
        output = UnitTest(self.vm_text).create_tests()
        if print_result:
            print()
            print(output)
            print()
        pasteboard_write(output)
        print('The result has been copied to the pasteboard.')



# coding=utf-8

from subprocess import call

from .template import Template, ProjectInfo
from .pb import pasteboard_read
from .command import Command, CommandOption
from .encoder import Encoder
from .model import Model, Enum


class TemplateCommand(Command):
    def __init__(self, template_name, scene_name, options):
        super(TemplateCommand, self).__init__()
        self.template_name = template_name
        self.scene_name = scene_name
        self.options = CommandOption(options)

    def get_project_package_name(self):
        try:
            infile = open('pubspec.yaml', 'r')
            return infile.readline().replace('name:','').strip()
        except (IOError, OSError) as e:
            print("Please run this command in flutter folder.")
            exit(1)

    def create_files(self):
        if self.template_name == Template.TemplateType.BASE:
            template = Template.BaseTemplate(
                self.options,
                self.scene_name,
                self.get_project_package_name()
            )
            output_path = template.create_files()
        else:
            print("Invalid template type.")
            exit(1)



# coding=utf-8

from jinja2 import Environment, PackageLoader
from .str_helpers import lower_first_letter, upper_first_letter
from .pb import pasteboard_write
from .command import Command


class APICommand(Command):
    def __init__(self, api_name):
        super(APICommand, self).__init__()
        self.api_name = api_name

    def create_api(self, print_result):
        output = API(self.api_name).create_api()
        if print_result:
            print()
            print(output)
            print()
        pasteboard_write(output)
        print('The result has been copied to the pasteboard.')


class API(object):
    def __init__(self, api_name):
        super(API, self).__init__()
        self.api_name = api_name
        self.env = Environment(
            loader=PackageLoader('ptfgen_templates', 'commands'),
            trim_blocks=True,
            lstrip_blocks=True
        )

    def create_api(self):
        name = upper_first_letter(self.api_name)
        var_name = lower_first_letter(name)
        template = self.env.get_template("api.dart")
        content = template.render(
            name=name,
            var_name=var_name
        )
        return content



# coding=utf-8


class Command:

    TAB_LENGTH = "<15"

    @classmethod
    def description(cls):
        return ""

    @classmethod
    def name(cls):
        return ""

    @classmethod
    def class_name(cls):
        return cls.__name__

    @classmethod
    def long_description(cls):
        return (format("   " + cls.name(), Command.TAB_LENGTH)
                + cls.description())

    @classmethod
    def help(cls):
        return cls.long_description()


class CommandOption:

    def __init__(self, options):
        self._options = options

    def __getattr__(self, name):
        cls = type(self)
        if name in self._options:
            return self._options[name]
        else:
            msg = '{.__name__!r} object has no attribute {!r}'
            raise AttributeError(msg.format(cls, name))



# coding=utf-8

import re
from jinja2 import Environment, PackageLoader
from .constants import (DART_TYPES_DEFAULT_VALUES,
                        DART_TYPES_MOCK_VALUES,
                        DART_TYPES)
from .str_helpers import upper_first_letter, lower_first_letter, is_number


class Model(object):

    PROPERTY_PATTERN = r'(?:let|var) (\w+)(?:: ([^=\s]*))?(?: = ([^/\n]+))?(?:\s*//.*)?'
    MODEL_PATTERN = r'(?:class|extension) (\w+)(?::\s)*(?:\w+,?\s?)* {([^}]+)'
    ENUM_PATTERN = r'(\w+)\.(\w+)'
    DIC_PATTERN = r'^\[\w*:\s?.*\]$'

    class Property(object):
        def __init__(self, property):
            self.property = property
            property_regex = re.compile(Model.PROPERTY_PATTERN)
            mo = property_regex.search(property)
            self.name = mo.group(1)
            self.name_title = upper_first_letter(self.name)
            self.type = Model.PropertyType(mo.group(2), mo.group(3))

        @property
        def is_url(self):
            lowered_name = self.name.lower()

            if lowered_name.endswith("id"):
                return False

            return "image" in lowered_name or "url" in lowered_name

    class PropertyType(object):
        def __init__(self, name, value):
            self.custom_types = {}

            if name is None:
                if value.startswith('"'):
                    self.name = 'String'
                elif value.endswith('()'):
                    self.name = value[:-2]
                elif value == 'true' or value == 'false':
                    self.name = 'Bool'
                elif is_number(value):
                    if '.' in value:
                        self.name = 'Double'
                    else:
                        self.name = 'Int'
                else:
                    mo = re.search(Model.ENUM_PATTERN, value)
                    if mo:
                        self.name = mo.group(1)
                        self.custom_types[self.name] = value
                    else:
                        self.name = 'Any'
            else:
                self.name = name

        @property
        def is_optional(self):
            return self.name.startswith('Optional<') or self.name.endswith("?")

        @property
        def is_array(self):
            return self.name.startswith('Array<') or (self.name.endswith("]") and ':' not in self.name)

        @property
        def is_dictionary(self):
            return self.name.startswith('Dictionary<') or (self.name.endswith("]") and ':' in self.name)

        @property
        def is_stream(self):
            return self.name.startswith('Stream<')

        @property
        def default_value(self):
            if self.is_optional:
                value = "nil"
            elif self.is_array:
                value = "[]"
            elif self.is_dictionary:
                value = "[:]"
            elif self.name in DART_TYPES:
                value = DART_TYPES_DEFAULT_VALUES[self.name]
            elif self.name in self.custom_types:
                value = self.custom_types[self.name]
            elif self.is_stream:
                value = 'Stream.empty()'
            else:
                value = "{}()".format(self.name)
            return value

        @property
        def mock_value(self):
            if self.name in DART_TYPES:
                return DART_TYPES_MOCK_VALUES[self.name]
            else:
                return self.default_value

    def __init__(self, name, properties):
        self.name = name
        self.properties = properties

    @classmethod
    def from_string(cls, string):
        model_regex = re.compile(Model.MODEL_PATTERN)
        match = model_regex.search(string)
        model_name = match.group(1)
        property_block = match.group(2)
        property_regex = re.compile(Model.PROPERTY_PATTERN)

        properties = [Model.Property(m.group())
                      for m in property_regex.finditer(property_block)]

        return cls(model_name, properties)


class InitModel(Model):

    def create_init(self):
        env = Environment(
            loader=PackageLoader('ptfgen_templates', 'commands'),
            trim_blocks=True,
            lstrip_blocks=True
        )

        template = env.get_template('init.dart')

        content = template.render(
            name=self.name,
            properties=self.properties
        )

        return content


class Enum(object):
    def __init__(self, name, cases):
        self.name = name
        self.name_variable = lower_first_letter(name)
        self.cases = cases
        self.cases_title = [upper_first_letter(c) for c in cases]
        self.case_count = len(cases)

    @classmethod
    def from_string(cls, enum_string):
        enum_regex = re.compile(r'enum (\w+)[^{]* {([\d\D]*)}')
        match = enum_regex.search(enum_string)
        enum_name = match.group(1)
        enum_block = match.group(2)
        regex = re.compile(r'case (\w+)')
        cases = [m.group(1) for m in regex.finditer(enum_block)]
        return cls(enum_name, cases)



# coding=utf-8

DART_TYPES_DEFAULT_VALUES = {
    'int': '0',
    'bool': 'false',
    'String': '""',
    'double': '0.0',
    'float': 'Float(0.0)',
    'date': 'DateTime()',
    'void': '()',
    'dynamic': 'null',
    'ValidationResult': 'ValidationResult.valid()'
}

DART_TYPES_MOCK_VALUES = {
    'Int': '1',
    'Bool': 'true',
    'String': '"foobar"',
    'Double': '1.0',
    'Float': 'Float(1.0)',
    'Date': 'DateTime()',
    'Void': '()',
    'dynamic': 'null',
    'ValidationResult': 'ValidationResult.valid()'
}

DART_TYPES = {
    'int',
    'bool',
    'String',
    'double',
    'float',
    'Date',
    'void',
    'dynamic',
    'ValidationResult'
}



# PYTHON_ARGCOMPLETE_OK

import sys
from arghandler import subcmd, ArgumentHandler

from . import __version__
from .pb import pasteboard_read
from .api_cmd import APICommand
from .mock_cmd import MockCommand
from .json_cmd import JSONCommand
from .test_cmd import UnitTestCommand
from .bind_cmd import BindViewModelCommand
from .template_cmd import TemplateCommand


@subcmd('template', help='create template files for a scene')
def cmd_template(parser, context, args):
    parser.description = 'Create template files for a scene.'

    parser.add_argument(
        'type',
        nargs=1,
        choices=[
            'base'
        ],
        help="template type"
    )

    parser.add_argument(
        'name',
        nargs=1,
        help='scene name'
    )

    args = parser.parse_args(args)
    template_name = args.type[0]
    scene_name = args.name[0]

    options = {}

    TemplateCommand(template_name, scene_name, options).create_files()


@subcmd('mock', help='create a mock class for a protocol/function')
def cmd_mock(parser, context, args):
    parser.usage = '''copy the protocol/function to the pasteboard then run: ptfgen mock [-h] [-p]'''
    parser.description = 'Create a mock class for a protocol/function.'
    parser.add_argument(
        '-p', '--print',
        required=False,
        action='store_true',
        help="print the result"
    )
    args = parser.parse_args(args)
    protocol_text = pasteboard_read()
    MockCommand(protocol_text).create_mock(args.print)


@subcmd('test', help='create unit tests for a view model')
def cmd_test(parser, context, args):
    parser.usage = '''copy the view model to the pasteboard then run: ptfgen test [-h] [-p]'''
    parser.description = 'Create unit tests for a view model.'
    parser.add_argument(
        '-p', '--print',
        required=False,
        action='store_true',
        help="print the result"
    )
    args = parser.parse_args(args)
    vm_text = pasteboard_read()
    UnitTestCommand(vm_text).create_tests(args.print)


@subcmd('bind', help='create a bindViewModel method for a UIViewController')
def cmd_bind(parser, context, args):
    parser.usage = '''copy the view model to the pasteboard then run: ptfgen bind [-h] [-p]'''
    parser.description = '''Create a bindViewModel method for a UIViewController.'''
    parser.add_argument(
        '-p', '--print',
        required=False,
        action='store_true',
        help="print the result"
    )
    args = parser.parse_args(args)
    vm_text = pasteboard_read()
    BindViewModelCommand(vm_text).create_bind_view_model(args.print)


@subcmd('json', help='create a model from JSON')
def cmd_json(parser, context, args):
    parser.usage = '''copy the JSON to the pasteboard then run: ptfgen json [-h] [-p] name'''
    parser.description = 'Create a model from JSON.'
    parser.add_argument(
        'name',
        nargs=1,
        help='model name'
    )
    parser.add_argument(
        '-p', '--print',
        required=False,
        action='store_true',
        help="print the result"
    )
    args = parser.parse_args(args)
    json = pasteboard_read()
    JSONCommand(args.name[0], json).create_models(args.print)


@subcmd('api', help='create input and output files for an API')
def cmd_api(parser, context, args):
    parser.description = 'Create input and output files for an API.'
    parser.add_argument(
        'name',
        nargs=1,
        help='api name'
    )
    parser.add_argument(
        '-p', '--print',
        required=False,
        action='store_true',
        help="print the result"
    )
    args = parser.parse_args(args)
    APICommand(args.name[0]).create_api(args.print)


def main():
    handler = ArgumentHandler(
        use_subcommand_help=True,
        enable_autocompletion=True,
        epilog='Get help on a subcommand: ptfgen subcommand -h'
    )

    handler.add_argument(
        '-v', '--version',
        action='version',
        version=__version__,
        help='show the version number'
    )

    # if no parameters are provided, show help
    if len(sys.argv) == 1:
        handler.run(['-h'])
    else:
        handler.run()


if __name__ == '__main__':
    main()



# coding=utf-8

import re
from jinja2 import Environment, PackageLoader
from .pb import pasteboard_write
from .command import Command
from .constants import DART_TYPES_DEFAULT_VALUES, DART_TYPES
from .str_helpers import upper_first_letter


class MockCommand(Command):
    def __init__(self, abstract_text):
        super(MockCommand, self).__init__()
        self.abstract_text = abstract_text

    def create_mock(self, print_result):
        output = Mock(self.abstract_text).create_mock()
        if print_result:
            print()
            print(output)
            print()
        pasteboard_write(output)
        print('The result has been copied to the pasteboard.')


class Mock(object):

    class Function(object):
        def __init__(self, origin, name, params, return_type):
            super(Mock.Function, self).__init__()
            self.origin = origin
            self.name = name
            self.params = list(filter(None, params.split(',')))
            self.return_type = return_type
            self.is_overloaded = False

        def __str__(self):
            return self.origin

        @property
        def return_value(self):
            if self.return_type is None:
                return_value = '()'
            if self.return_type == 'void':
                return_value = '()'
            elif self.return_type.endswith('?'):
                return_value = 'nil'
            elif self.return_type.startswith('Stream'):
                regex = re.compile('Stream<(.+)>')
                mo = regex.search(self.return_type)
                stream_type = mo.group(1)
                return_value = 'Stream<{}>.value(null)'.format(
                    stream_type
                )
            elif self.return_type in DART_TYPES:
                return_value = DART_TYPES_DEFAULT_VALUES[self.return_type]
            else:
                return_value = '{}()'.format(self.return_type)
            return return_value

        @property
        def return_void(self):
            return self.return_type is None or self.return_type == 'void'

        @property
        def return_nil(self):
            return self.return_value == 'nil'

        @property
        def first_param(self):
            if self.params:
                param = self.params[0]
                param_name = param.split(':')[0].split(' ')
                return param_name[0] \
                    + "".join(x.title() for x in param_name[1:])
            return None

        @property
        def first_param_title(self):
            if self.first_param is not None:
                return upper_first_letter(self.first_param)
            return None

        @property
        def overloaded_name(self):
            if self.is_overloaded and (self.first_param_title is not None):
                return self.name + self.first_param_title
            return self.name

    def __init__(self, abstract_text):
        super(Mock, self).__init__()
        self.abstract_text = abstract_text

    def _get_abstract_name(self, str):
        regex = re.compile(r'abstract class (\w+)')
        mo = regex.search(str)
        abstract_name = mo.group(1)
        if abstract_name.endswith('Type'):
            class_name = abstract_name[:-4]
        else:
            class_name = abstract_name
        return (abstract_name, class_name)

    def create_mock(self):
        str = self.abstract_text
        is_abstract = False
        abstract_name = ''
        class_name = ''
        # get abstract name
        try:
            (abstract_name, class_name) = self._get_abstract_name(str)
            is_abstract = True
        except Exception:
            pass
        # get functions
        func_regex = re.compile(r'  (.*) (.*)\((.*)\)')
        funcs = [Mock.Function(f.group(), f.group(2), f.group(3), f.group(1))
                 for f in func_regex.finditer(str)]

        if not funcs:
            print('The abstract or functions in the pasteboard is invalid.')
            exit(1)

        # check if overloaded
        func_dict = {}
        for f in funcs:
            if f.name in func_dict:
                func_dict[f.name] = func_dict[f.name] + 1
            else:
                func_dict[f.name] = 1
        for f in funcs:
            f.is_overloaded = func_dict[f.name] > 1

        env = Environment(
            loader=PackageLoader('ptfgen_templates', 'commands'),
            trim_blocks=True,
            lstrip_blocks=True
        )
        template = env.get_template("mock.dart")
        content = template.render(
            class_name=class_name,
            abstract_name=abstract_name,
            functions=funcs,
            is_abstract=is_abstract
        )
        return content



# coding=utf-8


def lower_first_letter(st):
    return st[0].lower() + st[1:]


def upper_first_letter(st):
    return st[0].upper() + st[1:]


def snake_to_camel(st):
    components = st.split('_')
    return components[0] + "".join(x.title() for x in components[1:])


def plural_to_singular(st):
    if st.endswith("ies"):
        if len(st) > 3:
            if st[-4] not in "aeiou":
                return st[0:-3] + "y"
    elif st.endswith("es"):
        if st[-3] in "sxo":
            return st[0:-2]
        elif st[-4:-2] == "ch" or st[-4:-2] == "sh":
            return st[0:-2]
        else:
            return st[0:-1]
    elif st.endswith("s"):
        if len(st) > 3:
            return st[:-1]
    return st


def is_number(st):
    try:
        float(st)
        return True
    except ValueError:
        pass

    try:
        import unicodedata
        unicodedata.numeric(st)
        return True
    except (TypeError, ValueError):
        pass

    return False



from setuptools import setup
from ptfgen import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='ptfgen',
    version=__version__,
    author="Pham Xuan Tien",
    author_email="tienpx.xxx@gmail.com",
    description="A code generator for Flutter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tienpx-x/PTfGen",
    license='MIT',
    packages=['ptfgen', 'ptfgen_templates'],
    entry_points={
        'console_scripts': [
            'ptfgen = ptfgen.__main__:main'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3",
    install_requires=['Jinja2>=2.10', 'arghandler>=1.2'],
    include_package_data=True
)



# coding=utf-8

import json
import re
from collections import OrderedDict
from jinja2 import Environment, PackageLoader
from .pb import pasteboard_write
from .constants import DART_TYPES_DEFAULT_VALUES, DART_TYPES
from .str_helpers import snake_to_camel, plural_to_singular
from .command import Command


class JSONCommand(Command):
    def __init__(self, model_name, json_text):
        super(JSONCommand, self).__init__()
        self.model_name = model_name
        self.json_text = json_text

    def create_models(self, print_result):
        output = JSON(self.model_name, self.json_text).create_models()
        if print_result:
            print()
            print(output)
            print()
        pasteboard_write(output)
        print('The result has been copied to the pasteboard.')


class JSON(object):

    IMPORT_HEADER = "import 'package:pt_object_mapper/pt_object_mapper.dart'; \n\n"; 

    JSON_TO_DART_TYPES = {
        'int': 'int',
        'bool': 'bool',
        'str': 'String',
        'float': 'double',
        'NoneType': 'dynamic'
    }

    DATE_REGEX = r'(\d{4})[-/](\d{2})[-/](\d{2})'

    class Property(object):
        def __init__(self, raw_name, name, type_name):
            self.raw_name = raw_name
            self.name = name
            self.type_name = type_name

        @property
        def is_user_type(self):
            return self.is_optional \
                   and self.original_type_name() not in DART_TYPES

        @property
        def is_optional(self):
            return self.type_name.endswith('?')

        @property
        def is_array(self):
            return self.type_name.startswith('[')

        @property
        def is_date(self):
            return self.original_type_name == 'DateTime'

        @property
        def original_type_name(self):
            return ''.join(c for c in self.type_name if c not in '?[]')

        @property
        def is_dart_type(self):
            return self.type_name in DART_TYPES_DEFAULT_VALUES

        @property
        def value(self):
            if self.is_optional:
                value = 'nil'
            elif self.is_array:
                value = '[]'
            elif self.type_name in DART_TYPES_DEFAULT_VALUES:
                value = DART_TYPES_DEFAULT_VALUES[self.type_name]
            else:
                value = '{}()'.format(self.type_name)
            return value

    class Model(object):
        def __init__(self, name, properties):
            self.name = name
            self.properties = properties

        def model(self):
            env = Environment(
                loader=PackageLoader('ptfgen_templates', 'commands'),
                trim_blocks=True,
                lstrip_blocks=True
            )
            template = env.get_template('json.dart')
            content = template.render(
                name=self.name,
                properties=self.properties
            )
            return content

    def __init__(self, model_name, json_text):
        self.model_name = model_name
        self.json_text = json_text

    def create_models(self):
        try:
            dictionary = json.loads(self.json_text,
                                    object_pairs_hook=OrderedDict)
            models = []
            self._extract_model(self.model_name, dictionary, models)
            output = '\n\n'.join([model.model() for model in models])
            output = JSON.IMPORT_HEADER + output
            return output
        except Exception:
            print('The JSON in the pasteboard is invalid.')
            exit(1)

    def _extract_model(self, name, dictionary, models):
        properties = []
        for key in dictionary.keys():
            var_name = snake_to_camel(key)
            value = dictionary[key]
            type_name = type(value).__name__
            if type_name == 'OrderedDict':
                var_type = var_name.title()
                self._extract_model(var_name.title(), value, models)
            elif type_name == 'list':
                singular_var_name = plural_to_singular(var_name)
                var_type = 'List<{}>'.format(singular_var_name.title())
                if len(value) > 0:
                    self._extract_model(
                        singular_var_name.title(),
                        value[0],
                        models
                    )
                else:
                    var_type = '[Any]'
            else:
                var_type = JSON.JSON_TO_DART_TYPES[type_name]
                if var_type == 'String':
                    if re.match(JSON.DATE_REGEX, value):
                        var_type = 'DateTime'
                if var_type == 'Any?':
                    if var_name.endswith('Url') \
                            or 'name' in var_name.lower() \
                            or 'email' in var_name.lower() \
                            or var_name.endswith('Key') \
                            or var_name.endswith('Token'):
                        var_type = 'String?'
                    elif 'time' in var_name.lower() \
                            or 'date' in var_name.lower() \
                            or var_name.endswith('At') \
                            or 'birthday' in var_name.lower():
                        var_type = 'DateTime?'
                    elif var_name.endswith('Id'):
                        var_type = 'Int?'
            property = JSON.Property(key, var_name, var_type)
            properties.append(property)
        model = JSON.Model(name, properties)
        models.append(model)
